@@ -1,6 +1,6 @@
 //===----------------------------------------------------------------------===//
 //
-// Copyright (c) 2012 The University of Utah
+// Copyright (c) 2012, 2013 The University of Utah
 // All rights reserved.
 //
 // This file is distributed under the University of Illinois Open Source
@@ -15,6 +15,7 @@
 #include "LocalToGlobal.h"
 
 #include "clang/AST/RecursiveASTVisitor.h"
+#include "clang/AST/ASTContext.h"
 #include "clang/Basic/SourceManager.h"
 
 #include "TransformationManager.h"
@@ -160,12 +161,9 @@ void LocalToGlobal::setNewName(FunctionDecl *FD,
 
 LocalToGlobal::~LocalToGlobal(void)
 {
-  if (FunctionVisitor)
-    delete FunctionVisitor;
-  if (LocalVarCollectionVisitor)
-    delete LocalVarCollectionVisitor;
-  if (TransformationASTVisitor)
-    delete TransformationASTVisitor;
+  delete FunctionVisitor;
+  delete LocalVarCollectionVisitor;
+  delete TransformationASTVisitor;
 }
 
 bool LToGASTVisitor::makeLocalAsGlobalVar(FunctionDecl *FD,
