@@ -1,6 +1,6 @@
 //===----------------------------------------------------------------------===//
 //
-// Copyright (c) 2012 The University of Utah
+// Copyright (c) 2012, 2013 The University of Utah
 // All rights reserved.
 //
 // This file is distributed under the University of Illinois Open Source
@@ -15,7 +15,9 @@
 #include "ReduceClassTemplateParameter.h"
 
 #include "llvm/ADT/DenseMap.h"
+#include "llvm/ADT/SmallPtrSet.h"
 #include "clang/AST/RecursiveASTVisitor.h"
+#include "clang/AST/ASTContext.h"
 #include "clang/Basic/SourceManager.h"
 
 #include "TransformationManager.h"
@@ -51,37 +53,30 @@ private:
 
 namespace {
 
+typedef SmallPtrSet<const NamedDecl *, 8> TemplateParameterSet;
+
 class TemplateParameterVisitor : public 
   RecursiveASTVisitor<TemplateParameterVisitor> {
 
 public:
-  TemplateParameterVisitor(void)
-             : UsedParameters(NULL) { 
-    UsedParameters = new TemplateParameterSet();
-  }
+  explicit TemplateParameterVisitor(TemplateParameterSet &Params)
+             : UsedParameters(Params) 
+  { }
 
-  ~TemplateParameterVisitor() {
-    delete UsedParameters;
-  };
+  ~TemplateParameterVisitor() { };
 
   bool VisitTemplateTypeParmType(TemplateTypeParmType *Ty);
 
-  bool isUsedParam(const NamedDecl *ND) {
-    return UsedParameters->count(ND);
-  }
-
 private:
 
-  typedef SmallPtrSet<const NamedDecl *, 8> TemplateParameterSet;
-
-  TemplateParameterSet *UsedParameters;
+  TemplateParameterSet &UsedParameters;
 };
 
 bool TemplateParameterVisitor::VisitTemplateTypeParmType(
        TemplateTypeParmType *Ty)
 {
   const TemplateTypeParmDecl *D = Ty->getDecl();
-  UsedParameters->insert(D);
+  UsedParameters.insert(D);
   return true;
 }
 
@@ -186,9 +181,11 @@ bool ReduceClassTemplateParameterASTVisitor::VisitClassTemplateDecl(
   if (!ConsumerInstance->isValidClassTemplateDecl(D))
     return true;
 
-  TemplateParameterVisitor ParameterVisitor;
+  TemplateParameterSet ParamsSet;
+  TemplateParameterVisitor ParameterVisitor(ParamsSet);
   CXXRecordDecl *CXXRD = D->getTemplatedDecl();
-  if (CXXRecordDecl *Def = CXXRD->getDefinition())
+  CXXRecordDecl *Def = CXXRD->getDefinition();
+  if (Def)
     ParameterVisitor.TraverseDecl(Def);
 
   // ISSUE: we should also check the parameter usage for partial template
@@ -200,12 +197,22 @@ bool ReduceClassTemplateParameterASTVisitor::VisitClassTemplateDecl(
   //   template<bool, typename T> struct S{};
   //   template<typename T> struct<true, T> S{};
   // if we remove bool and true, we will have two definitions for S
+  TemplateParameterList *TPList;
+  if (Def) {
+    // make sure we use the params as in ParameterVisitor
+    const ClassTemplateDecl *CT = Def->getDescribedClassTemplate();
+    TransAssert(CT && "NULL DescribedClassTemplate!");
+    TPList = CT->getTemplateParameters();
+  }
+  else {
+    TPList = CanonicalD->getTemplateParameters();
+  }
+
   unsigned Index = 0;
-  TemplateParameterList *TPList = CanonicalD->getTemplateParameters();
   for (TemplateParameterList::const_iterator I = TPList->begin(),
        E = TPList->end(); I != E; ++I) {
     const NamedDecl *ND = (*I);
-    if (ParameterVisitor.isUsedParam(ND)) {
+    if (ParamsSet.count(ND)) {
       Index++;
       continue;
     }
@@ -283,11 +290,12 @@ void ReduceClassTemplateParameter::removeParameterByRange(SourceRange Range,
   }
 }
 
-void ReduceClassTemplateParameter::removeParameterFromDecl(void)
+void ReduceClassTemplateParameter::removeParameterFromDecl()
 {
   unsigned NumParams = TheClassTemplateDecl->getTemplateParameters()->size();
   
-  TransAssert((NumParams > 1) && "Bad size of TheClassTemplateDecl!");
+  TransAssert((NumParams > 1) && "Bad size of TheClassTemplateDecl!"); 
+  (void)NumParams;
 
   for (ClassTemplateDecl::redecl_iterator 
          I = TheClassTemplateDecl->redecls_begin(), 
@@ -326,6 +334,7 @@ void ReduceClassTemplateParameter::removeOneParameterByArgExpression(
 
   unsigned NumParams = TPList->size();
   TransAssert((Idx < NumParams) && "Cannot find valid TemplateParameter!");
+  (void)NumParams;
   SourceRange Range = ParmD->getSourceRange();
   removeParameterByRange(Range, TPList, Idx);
 }
@@ -431,6 +440,7 @@ void ReduceClassTemplateParameter::removeOneParameterByArgTemplate(
 
   unsigned NumParams = TPList->size();
   TransAssert((Idx < NumParams) && "Cannot find valid TemplateParameter!");
+  (void)NumParams;
   SourceRange Range = TmplD->getSourceRange();
   removeParameterByRange(Range, TPList, Idx);
   
@@ -584,7 +594,7 @@ bool ReduceClassTemplateParameter::reducePartialSpec(
 // ISSUE: The transformation is known to go wrong in the following case:
 // template<typename T1, typename T2> struct S;
 // template<typename T1, typename T2> struct S<T2, T1>;
-void ReduceClassTemplateParameter::removeParameterFromPartialSpecs(void)
+void ReduceClassTemplateParameter::removeParameterFromPartialSpecs()
 {
   SmallVector<ClassTemplatePartialSpecializationDecl *, 10> PartialDecls;
   TheClassTemplateDecl->getPartialSpecializations(PartialDecls);
@@ -681,13 +691,10 @@ bool ReduceClassTemplateParameter::referToTheTemplateDecl(
   return Context->hasSameTemplateName(*TheTemplateName, TmplName);
 }
 
-ReduceClassTemplateParameter::~ReduceClassTemplateParameter(void)
+ReduceClassTemplateParameter::~ReduceClassTemplateParameter()
 {
-  if (TheTemplateName)
-    delete TheTemplateName;
-  if (CollectionVisitor)
-    delete CollectionVisitor;
-  if (ArgRewriteVisitor)
-    delete ArgRewriteVisitor;
+  delete TheTemplateName;
+  delete CollectionVisitor;
+  delete ArgRewriteVisitor;
 }
 
