@@ -1,6 +1,6 @@
 //===----------------------------------------------------------------------===//
 //
-// Copyright (c) 2012 The University of Utah
+// Copyright (c) 2012, 2013 The University of Utah
 // All rights reserved.
 //
 // This file is distributed under the University of Illinois Open Source
@@ -15,6 +15,7 @@
 #include "ReducePointerLevel.h"
 
 #include "clang/AST/RecursiveASTVisitor.h"
+#include "clang/AST/ASTContext.h"
 #include "clang/Basic/SourceManager.h"
 
 #include "TransformationManager.h"
@@ -171,11 +172,8 @@ bool PointerLevelCollectionVisitor::VisitUnaryOperator(UnaryOperator *UO)
       !dyn_cast<ArraySubscriptExpr>(SubE))
     return true;
 
-  const DeclaratorDecl *DD = 
-    ConsumerInstance->getRefDecl(SubE);
-  TransAssert(DD && "NULL DD!");
-
-  ConsumerInstance->AddrTakenDecls.insert(DD);
+  if (const DeclaratorDecl *DD = ConsumerInstance->getRefDecl(SubE))
+    ConsumerInstance->AddrTakenDecls.insert(DD);
   return true;
 }
 
@@ -193,7 +191,8 @@ bool PointerLevelCollectionVisitor::VisitBinaryOperator(BinaryOperator *BO)
   if (dyn_cast<DeclRefExpr>(Rhs) || 
       dyn_cast<UnaryOperator>(Rhs) ||
       dyn_cast<ArraySubscriptExpr>(Rhs) ||
-      dyn_cast<MemberExpr>(Rhs))
+      dyn_cast<MemberExpr>(Rhs) ||
+      dyn_cast<IntegerLiteral>(Rhs))
     return true;
 
   const DeclaratorDecl *DD = ConsumerInstance->getRefDecl(Lhs);
@@ -299,6 +298,10 @@ bool PointerLevelRewriteVisitor::VisitBinaryOperator(BinaryOperator *BO)
     return true;
 
   const Expr *Lhs = BO->getLHS();
+  // Lhs could be CallExpr
+  if (dyn_cast<CallExpr>(Lhs) || dyn_cast<CXXConstructExpr>(Lhs) ||
+      dyn_cast<CXXUnresolvedConstructExpr>(Lhs))
+    return true;
   const DeclaratorDecl *DD = ConsumerInstance->getRefDecl(Lhs);
   TransAssert(DD && "NULL DD!");
   if (DD != ConsumerInstance->TheDecl)
@@ -529,6 +532,9 @@ const DeclaratorDecl *ReducePointerLevel::getRefDecl(const Expr *Exp)
 {
   const Expr *E = ignoreSubscriptExprParenCasts(Exp);
 
+  if (dyn_cast<CXXThisExpr>(E))
+    return NULL;
+
   if (const DeclRefExpr *DRE = dyn_cast<DeclRefExpr>(E))
     return getCanonicalDeclaratorDecl(DRE);
 
@@ -536,7 +542,11 @@ const DeclaratorDecl *ReducePointerLevel::getRefDecl(const Expr *Exp)
     return getCanonicalDeclaratorDecl(ME);
 
   const UnaryOperator *UO = dyn_cast<UnaryOperator>(E);
-  TransAssert(UO && "Bad UnaryOperator!");
+  // In some case, E could not be of UO if the program under transformation 
+  // is invalid.
+  if (!UO)
+    return NULL;
+
   UnaryOperator::Opcode Op = UO->getOpcode();
   (void)Op;
   TransAssert(((Op == UO_Deref) || (Op == UO_AddrOf)) && 
@@ -609,6 +619,11 @@ void ReducePointerLevel::copyInitStr(const Expr *Exp,
   case Expr::DeclRefExprClass: {
     const DeclRefExpr *DRE = dyn_cast<DeclRefExpr>(E);
     const ValueDecl *OrigDecl = DRE->getDecl();
+    if (dyn_cast<FunctionDecl>(OrigDecl)) {
+      InitStr = "0";
+      return;
+    }
+
     const VarDecl *VD = dyn_cast<VarDecl>(OrigDecl);
     TransAssert(VD && "Bad VarDecl!");
     const Expr *InitE = VD->getAnyInitializer();
@@ -800,6 +815,7 @@ void ReducePointerLevel::getNewLocalInitStr(const Expr *Init,
   }
 
   case Expr::MemberExprClass: // Fall-through
+  case Expr::BinaryOperatorClass: 
   case Expr::ArraySubscriptExprClass: {
     RewriteHelper->getExprString(E, InitStr);
     InitStr = "*(" + InitStr + ")";
@@ -812,8 +828,9 @@ void ReducePointerLevel::getNewLocalInitStr(const Expr *Init,
                           &ReducePointerLevel::getNewLocalInitStr);
     return; 
   }
+
   default:
-    TransAssert(0 && "Uncatched initializer!");
+    TransAssert(0 && "Uncaught initializer!");
   }
 
   TransAssert(0 && "Unreachable code!");
@@ -926,10 +943,8 @@ void ReducePointerLevel::rewriteArrayInit(const RecordDecl *RD,
 
 ReducePointerLevel::~ReducePointerLevel(void)
 {
-  if (CollectionVisitor)
-    delete CollectionVisitor;
-  if (RewriteVisitor)
-    delete RewriteVisitor;
+  delete CollectionVisitor;
+  delete RewriteVisitor;
 
   for (LevelToDeclMap::iterator I = AllPtrDecls.begin(), 
        E = AllPtrDecls.end(); I != E; ++I) {
