@@ -1,6 +1,6 @@
 //===----------------------------------------------------------------------===//
 //
-// Copyright (c) 2012 The University of Utah
+// Copyright (c) 2012, 2013 The University of Utah
 // All rights reserved.
 //
 // This file is distributed under the University of Illinois Open Source
@@ -16,6 +16,7 @@
 
 #include <sstream>
 #include "clang/AST/RecursiveASTVisitor.h"
+#include "clang/AST/ASTContext.h"
 #include "clang/Basic/SourceManager.h"
 
 #include "TransformationManager.h"
@@ -239,9 +240,19 @@ bool RemoveNamespaceRewriteVisitor::VisitNamespaceAliasDecl(
 
   const NamespaceDecl *CanonicalND = 
     D->getNamespace()->getCanonicalDecl();
-  if (CanonicalND == ConsumerInstance->TheNamespaceDecl) {
-    ConsumerInstance->RewriteHelper->removeDecl(D);
-    return true;
+  if (D->getQualifier()) {
+    TraverseNestedNameSpecifierLoc(D->getQualifierLoc());
+    if (CanonicalND == ConsumerInstance->TheNamespaceDecl) {
+      NestedNameSpecifierLoc QualLoc = D->getQualifierLoc();
+      SourceLocation QualEndLoc = QualLoc.getEndLoc();
+      SourceLocation DeclEndLoc = D->getSourceRange().getEnd();
+      ConsumerInstance->TheRewriter.RemoveText(
+        SourceRange(QualEndLoc, DeclEndLoc));
+    }
+  }
+  else {
+    if (CanonicalND == ConsumerInstance->TheNamespaceDecl)
+      ConsumerInstance->RewriteHelper->removeDecl(D);
   }
 
   return true;
@@ -545,7 +556,7 @@ bool RemoveNamespaceRewriteVisitor::TraverseNestedNameSpecifierLoc(
     return true;
   }
 
-  if (!QualifierLoc)
+  if (!QualifierLoc || QualifierLoc.getBeginLoc().isInvalid())
     return true;
 
   SmallVector<NestedNameSpecifierLoc, 8> QualifierLocs;
@@ -565,7 +576,8 @@ bool RemoveNamespaceRewriteVisitor::TraverseNestedNameSpecifierLoc(
       }
       case NestedNameSpecifier::NamespaceAlias: {
         const NamespaceAliasDecl *NAD = NNS->getAsNamespaceAlias();
-        ND = NAD->getNamespace()->getCanonicalDecl();
+        if (!NAD->getQualifier())
+          ND = NAD->getNamespace()->getCanonicalDecl();
         break;
       }
       case NestedNameSpecifier::TypeSpec: // Fall-through
@@ -731,7 +743,7 @@ bool RemoveNamespace::hasNameConflict(const NamedDecl *ND,
 
   DeclarationName Name = ND->getDeclName();
   DeclContextLookupConstResult Result = ParentCtx->lookup(Name);
-  return (Result.first != Result.second);
+  return !Result.empty();
 }
 
 // We always prepend the Prefix string to EnumConstantDecl if ParentCtx
@@ -1075,6 +1087,9 @@ bool RemoveNamespace::isTheNamespaceSpecifier(const NestedNameSpecifier *NNS)
 
   case NestedNameSpecifier::NamespaceAlias: {
     const NamespaceAliasDecl *NAD = NNS->getAsNamespaceAlias();
+    // we remove the namealias only when it doesn't have nestedspecifier
+    if (NAD->getQualifier())
+      return false;
     const NamespaceDecl *CanonicalND = 
       NAD->getNamespace()->getCanonicalDecl();
     return (CanonicalND == TheNamespaceDecl);
@@ -1142,9 +1157,7 @@ void RemoveNamespace::removeLastNamespaceFromUsingDecl(
 
 RemoveNamespace::~RemoveNamespace(void)
 {
-  if (CollectionVisitor)
-    delete CollectionVisitor;
-  if (RewriteVisitor)
-    delete RewriteVisitor;
+  delete CollectionVisitor;
+  delete RewriteVisitor;
 }
 
