@@ -1,6 +1,6 @@
 //===----------------------------------------------------------------------===//
 //
-// Copyright (c) 2012 The University of Utah
+// Copyright (c) 2012, 2013 The University of Utah
 // All rights reserved.
 //
 // This file is distributed under the University of Illinois Open Source
@@ -18,6 +18,7 @@
 
 #include "llvm/ADT/SmallVector.h"
 #include "clang/AST/RecursiveASTVisitor.h"
+#include "clang/AST/ASTContext.h"
 #include "clang/Basic/SourceManager.h"
 
 #include "TransformationManager.h"
@@ -61,6 +62,8 @@ public:
 
   bool VisitCallExpr(CallExpr *CallE);
 
+  bool VisitStmtExpr(StmtExpr *SE);
+
 private:
   RemoveNestedFunction *ConsumerInstance;
 
@@ -77,6 +80,24 @@ bool RNFCollectionVisitor::VisitFunctionDecl(FunctionDecl *FD)
   return true;
 }
 
+bool RNFStatementVisitor::VisitStmtExpr(StmtExpr *SE)
+{
+  CompoundStmt *CS = SE->getSubStmt();
+  if (ConsumerInstance->CallExprQueue.empty()) {
+    TraverseStmt(CS);
+    return false;
+  }
+
+  CallExpr *CallE = ConsumerInstance->CallExprQueue.back();
+  CurrentStmt = CallE;
+  
+  for (clang::CompoundStmt::body_iterator I = CS->body_begin(),
+       E = CS->body_end(); I != E; ++I) {
+    TraverseStmt(*I);
+  }
+  return false;
+}
+
 bool RNFStatementVisitor::VisitCallExpr(CallExpr *CallE) 
 {
   if (const CXXOperatorCallExpr *OpE = dyn_cast<CXXOperatorCallExpr>(CallE)) {
@@ -105,7 +126,7 @@ bool RNFStatementVisitor::VisitCallExpr(CallExpr *CallE)
 
   for (CallExpr::arg_iterator I = CallE->arg_begin(),
        E = CallE->arg_end(); I != E; ++I) {
-      TraverseStmt(*I);
+    TraverseStmt(*I);
   }
 
   ConsumerInstance->CallExprQueue.pop_back();
@@ -148,7 +169,7 @@ void RemoveNestedFunction::HandleTranslationUnit(ASTContext &Ctx)
 
   NameQueryWrap->TraverseDecl(Ctx.getTranslationUnitDecl());
 
-  addNewTmpVariable();
+  addNewTmpVariable(Ctx);
   addNewAssignStmt();
   replaceCallExpr();
 
@@ -157,7 +178,45 @@ void RemoveNestedFunction::HandleTranslationUnit(ASTContext &Ctx)
     TransError = TransInternalError;
 }
 
-bool RemoveNestedFunction::addNewTmpVariable(void)
+void RemoveNestedFunction::getVarStrForTemplateSpecialization(
+       std::string &VarStr,
+       const TemplateSpecializationType *TST)
+{
+  unsigned NumArgs = TST->getNumArgs();
+  if (NumArgs == 0) {
+    VarStr += ";";
+    return;
+  }
+
+  std::string ArgStr;
+  llvm::raw_string_ostream Stream(ArgStr);
+  TST->getArg(0).print(Context->getPrintingPolicy(), Stream);
+
+  for (unsigned I = 1; I < NumArgs; ++I) {
+    const TemplateArgument &Arg = TST->getArg(I);
+    Stream << ", ";
+    Arg.print(Context->getPrintingPolicy(), Stream);
+  }
+  size_t BeginPos = VarStr.find_first_of('<');
+  size_t EndPos = VarStr.find_last_of('>');
+  TransAssert((BeginPos != std::string::npos) && "Cannot find < !");
+  TransAssert((EndPos != std::string::npos) && "Cannot find > !");
+  TransAssert((EndPos > BeginPos) && "Invalid <> pair!");
+  VarStr.replace(BeginPos + 1, (EndPos - BeginPos - 1), Stream.str());
+  VarStr += ";";
+}
+
+bool RemoveNestedFunction::writeNewTmpVariable(const QualType &QT,
+                                               std::string &VarStr)
+{
+  QT.getAsStringInternal(VarStr,
+                         Context->getPrintingPolicy());
+
+  VarStr += ";";
+  return RewriteHelper->addLocalVarToFunc(VarStr, TheFuncDecl);
+}
+
+bool RemoveNestedFunction::addNewTmpVariable(ASTContext &ASTCtx)
 {
   std::string VarStr;
   std::stringstream SS;
@@ -170,7 +229,6 @@ bool RemoveNestedFunction::addNewTmpVariable(void)
   QualType QT;
   const Expr *E = TheCallExpr->getCallee();
 
-  // ISSUE: also handle UnresolvedMemberExpr? 
   if (const UnresolvedLookupExpr *UE = dyn_cast<UnresolvedLookupExpr>(E)) {
     // clang doesn't always resolve CallExpr's callee. For example:
     //   template<typename T> int foo1(int p) {return p;}
@@ -193,9 +251,31 @@ bool RemoveNestedFunction::addNewTmpVariable(void)
       FD = lookupFunctionDecl(DName, TheFuncDecl->getLookupParent());
     TransAssert(FD && "Cannot resolve DName!");
     QT = FD->getResultType();
+    return writeNewTmpVariable(QT, VarStr);
   }
-  else if (const CXXDependentScopeMemberExpr *ME = 
+
+  if (const UnresolvedMemberExpr *UM = dyn_cast<UnresolvedMemberExpr>(E)) {
+    DeclarationName DName = UM->getMemberName();
+    CXXRecordDecl *CXXRD = UM->getNamingClass();
+    const FunctionDecl *FD = lookupFunctionDecl(DName, CXXRD);
+    // FIXME: try to resolve FD here
+    if (FD)
+      QT = FD->getResultType();
+    return writeNewTmpVariable(QT, VarStr);
+  }
+
+  if (const CXXDependentScopeMemberExpr *ME = 
       dyn_cast<CXXDependentScopeMemberExpr>(E)) {
+
+    if (ME->isImplicitAccess())
+      return false;
+    DeclarationName DName = ME->getMember();
+    TransAssert((DName.getNameKind() == DeclarationName::Identifier) &&
+                "Not an indentifier!");
+    const Expr *E = ME->getBase();
+    TransAssert(E && "NULL Base Expr!");
+    const Expr *BaseE = E->IgnoreParens();
+
     // handle cases where base expr or member name is dependent, e.g.,
     // template<typename T>
     // class S {
@@ -209,25 +289,93 @@ bool RemoveNestedFunction::addNewTmpVariable(void)
     //   f1(this->f2(1));
     // }
     // where this->f2(1) is a CXXDependentScopeMemberExpr
+    if (dyn_cast<CXXThisExpr>(BaseE)) {
+      const DeclContext *Ctx = TheFuncDecl->getLookupParent();
+      TransAssert(Ctx && "Bad DeclContext!");
+      const FunctionDecl *FD = lookupFunctionDecl(DName, Ctx);
+      TransAssert(FD && "Cannot resolve DName!");
+      QT = FD->getResultType();
+      return writeNewTmpVariable(QT, VarStr);
+    }
     
-    DeclarationName DName = ME->getMember();
-    TransAssert((DName.getNameKind() == DeclarationName::Identifier) &&
-                "Not an indentifier!");
+    // handle other cases where lookupDeclContext is different from 
+    // the current CXXRecord, e.g.,
+    const Type *Ty = ME->getBaseType().getTypePtr();
+    if (const DeclContext *Ctx = getBaseDeclFromType(Ty)) {
+      const FunctionDecl *FD = lookupFunctionDecl(DName, Ctx);
+      if (!FD) {
+        return writeNewTmpVariable(QT, VarStr);
+      }
+      QT = FD->getResultType();
+      const Type *RVTy = QT.getTypePtr();
+      if (RVTy->getAs<InjectedClassNameType>()) {
+        // handle cases like:
+        // template <typename> struct D {
+        //   D f();
+        // };
+        // template <typename T> void foo(D<T>);
+        // template <typename T > void bar () {
+        //   D<T> G;
+        //   foo(G.f());
+        // }
+        // in this case, seems it's hard to retrieve the instantiated type
+        // of f's return type, because `D<T> G' is dependent. I tried
+        // findSpecialization from ClassTemplateDecl, but it didn't work. 
+        // So use a really ugly way, i.e., manipulating strings...
+        const TemplateSpecializationType *TST = 
+          Ty->getAs<TemplateSpecializationType>();
+        TransAssert(TST && "Invalid TemplateSpecialization Type!");
 
-    const FunctionDecl *FD = 
-      lookupFunctionDecl(DName, TheFuncDecl->getLookupParent());
-    TransAssert(FD && "Cannot resolve DName!");
-    QT = FD->getResultType();
-  }
-  else {
-    QT = TheCallExpr->getCallReturnType();
+        QT.getAsStringInternal(VarStr,
+                               Context->getPrintingPolicy());
+        getVarStrForTemplateSpecialization(VarStr, TST);
+        return RewriteHelper->addLocalVarToFunc(VarStr, TheFuncDecl);
+      }
+      else {
+        // other cases:
+        // template <typename> struct D {
+        //   int f();
+        // };
+        // void foo(int);
+        // template <typename T > void bar () {
+        //   D<T> G;
+        //   foo(G.f());
+        // }
+        return writeNewTmpVariable(QT, VarStr);
+      }
+    }
+    else {
+      // template <typename> struct D { 
+      // D f();
+      // D operator[] (int);
+      // };
+      // template <typename T> void foo(D<T>);
+      // template <typename T > void bar () {
+      //   D<T> G;
+      //   foo(G[0].f());
+      // }
+      // In this case, G[0] is of BuiltinType.
+      // But why does clang represent a dependent type as BuiltinType here?
+
+      TransAssert((Ty->getAs<BuiltinType>() || Ty->getAs<TemplateTypeParmType>())
+                  && "Uncaught Type");
+      // FIXME: This is incorrect!
+      // a couple of questions
+      //  - how can we find a correct DeclContext where we could lookup f?
+      //  - can we obtain the dependent template argument from BuiltinType?
+      // Probably we cannot do these? Comments from lib/AST/ASTContext.cpp:
+      // 
+      // Placeholder type for type-dependent expressions whose type is
+      // completely unknown. No code should ever check a type against
+      // DependentTy and users should never see it; however, it is here to
+      // help diagnose failures to properly check for type-dependent
+      // expressions.
+      return writeNewTmpVariable(QT, VarStr);
+    }
   }
 
-  QT.getAsStringInternal(VarStr,
-                         Context->getPrintingPolicy());
-
-  VarStr += ";";
-  return RewriteHelper->addLocalVarToFunc(VarStr, TheFuncDecl);
+  QT = TheCallExpr->getCallReturnType();
+  return writeNewTmpVariable(QT, VarStr);
 }
 
 bool RemoveNestedFunction::addNewAssignStmt(void)
@@ -251,6 +399,7 @@ bool RemoveNestedFunction::isInvalidOperator(const CXXOperatorCallExpr *OpE)
   //        omit it for now.
   switch (K) {
   case OO_Equal:
+  case OO_EqualEqual:
   case OO_PlusEqual:
   case OO_MinusEqual:
   case OO_StarEqual:
@@ -272,13 +421,8 @@ bool RemoveNestedFunction::isInvalidOperator(const CXXOperatorCallExpr *OpE)
 
 RemoveNestedFunction::~RemoveNestedFunction(void)
 {
-  if (NestedInvocationVisitor)
-    delete NestedInvocationVisitor;
-
-  if (StmtVisitor)
-    delete StmtVisitor;
-
-  if (NameQueryWrap)
-    delete NameQueryWrap;
+  delete NestedInvocationVisitor;
+  delete StmtVisitor;
+  delete NameQueryWrap;
 }
 
