@@ -1,6 +1,6 @@
 //===----------------------------------------------------------------------===//
 //
-// Copyright (c) 2012 The University of Utah
+// Copyright (c) 2012, 2013 The University of Utah
 // All rights reserved.
 //
 // This file is distributed under the University of Illinois Open Source
@@ -17,6 +17,7 @@
 #include <sstream>
 
 #include "clang/AST/RecursiveASTVisitor.h"
+#include "clang/AST/ASTContext.h"
 #include "clang/Basic/SourceManager.h"
 
 #include "TransformationManager.h"
@@ -591,8 +592,7 @@ void ReplaceCallExpr::replaceCallExpr(void)
 
 ReplaceCallExpr::~ReplaceCallExpr(void)
 {
-  if (CollectionVisitor)
-    delete CollectionVisitor;
+  delete CollectionVisitor;
 
   for (DenseMap<FunctionDecl *, ReturnStmtsVector *>::iterator 
        I = FuncToReturnStmts.begin(), E = FuncToReturnStmts.end();
