@@ -1,6 +1,6 @@
 //===----------------------------------------------------------------------===//
 //
-// Copyright (c) 2012 The University of Utah
+// Copyright (c) 2012, 2013 The University of Utah
 // All rights reserved.
 //
 // This file is distributed under the University of Illinois Open Source
@@ -17,7 +17,7 @@
 #include <cctype>
 #include <sstream>
 #include "clang/Basic/SourceManager.h"
-#include "clang/Rewrite/Rewriter.h"
+#include "clang/Rewrite/Core/Rewriter.h"
 #include "clang/AST/Decl.h"
 #include "clang/AST/DeclCXX.h"
 #include "clang/AST/DeclTemplate.h"
@@ -84,12 +84,17 @@ unsigned RewriteUtils::getOffsetBetweenLocations(SourceLocation StartLoc,
 
 SourceLocation RewriteUtils::getEndLocationFromBegin(SourceRange Range)
 {
+  SourceLocation StartLoc = Range.getBegin();
+  SourceLocation EndLoc = Range.getEnd();
+  if (StartLoc.isInvalid())
+    return StartLoc;
+  if (EndLoc.isInvalid())
+    return EndLoc;
+
   int LocRangeSize = TheRewriter->getRangeSize(Range);
   if (LocRangeSize == -1)
     return Range.getEnd();
 
-  SourceLocation StartLoc = Range.getBegin();
-
   return StartLoc.getLocWithOffset(LocRangeSize);
 }
 
@@ -121,6 +126,8 @@ SourceLocation RewriteUtils::getEndLocationUntil(SourceRange Range,
                                                  char Symbol)
 {
   SourceLocation EndLoc = getEndLocationFromBegin(Range);
+  if (EndLoc.isInvalid())
+    return EndLoc;
     
   const char *EndBuf = SrcManager->getCharacterData(EndLoc);
   int Offset = getOffsetUntil(EndBuf, Symbol);
@@ -139,6 +146,8 @@ SourceLocation RewriteUtils::getEndLocationAfter(SourceRange Range,
                                                 char Symbol)
 {
   SourceLocation EndLoc = getEndLocationFromBegin(Range);
+  if (EndLoc.isInvalid())
+    return EndLoc;
     
   const char *EndBuf = SrcManager->getCharacterData(EndLoc);
   int Offset = getOffsetUntil(EndBuf, Symbol);
@@ -493,6 +502,7 @@ SourceLocation RewriteUtils::getVarDeclTypeLocEnd(const VarDecl *VD)
 
   SourceRange TypeLocRange = VarTypeLoc.getSourceRange();
   SourceLocation EndLoc = getEndLocationFromBegin(TypeLocRange);
+  TransAssert(EndLoc.isValid() && "Invalid EndLoc!");
 
   const Type *Ty = VarTypeLoc.getTypePtr();
 
@@ -697,6 +707,7 @@ bool RewriteUtils::addNewAssignStmtBefore(Stmt *BeforeStmt,
     SourceRange StmtRange = BeforeStmt->getSourceRange();
     SourceLocation LocEnd = 
       RewriteUtils::getEndLocationFromBegin(StmtRange);
+    TransAssert(LocEnd.isValid() && "Invalid LocEnd!");
 
     std::string PostStr = "\n" + IndentStr + "}";
     if (TheRewriter->InsertTextAfterToken(LocEnd, PostStr))
@@ -750,6 +761,7 @@ bool RewriteUtils::addStringBeforeStmt(Stmt *BeforeStmt,
     SourceRange StmtRange = BeforeStmt->getSourceRange();
     SourceLocation LocEnd = 
       RewriteUtils::getEndLocationFromBegin(StmtRange);
+    TransAssert(LocEnd.isValid() && "Invalid LocEnd!");
 
     std::string PostStr = "\n" + IndentStr + "}";
     if (TheRewriter->InsertTextAfterToken(LocEnd, PostStr))
@@ -782,11 +794,12 @@ bool RewriteUtils::addStringAfterStmt(Stmt *AfterStmt,
   SourceRange StmtRange = AfterStmt->getSourceRange();
   SourceLocation LocEnd = 
     RewriteUtils::getEndLocationFromBegin(StmtRange);
+  TransAssert(LocEnd.isValid() && "Invalid LocEnd!");
   
   return !(TheRewriter->InsertText(LocEnd, NewStr));
 }
 
-bool RewriteUtils::addStringAfterVarDecl(VarDecl *VD,
+bool RewriteUtils::addStringAfterVarDecl(const VarDecl *VD,
                                          const std::string &Str)
 {
   SourceRange VarRange = VD->getSourceRange();
@@ -1045,7 +1058,9 @@ SourceLocation RewriteUtils::getDeclGroupRefEndLoc(DeclGroupRef DGR)
 
   // The LastD could be a RecordDecl
   SourceRange Range = LastD->getSourceRange();
-  return getEndLocationFromBegin(Range);
+  SourceLocation EndLoc = getEndLocationFromBegin(Range);
+  TransAssert(EndLoc.isValid() && "Invalid EndLoc!");
+  return EndLoc;
 }
 
 SourceLocation RewriteUtils::getDeclStmtEndLoc(DeclStmt *DS)
@@ -1134,6 +1149,10 @@ bool RewriteUtils::removeVarDecl(const VarDecl *VD,
   if (DGR.isSingleDecl()) {
     SourceLocation StartLoc = VarRange.getBegin();
     SourceLocation EndLoc = getEndLocationUntil(VarRange, ';');
+    // in case the previous EndLoc is invalid, for example,
+    // VarRange could have bad EndLoc value
+    if (EndLoc.isInvalid())
+      EndLoc = getLocationUntil(StartLoc, ';');
     return !(TheRewriter->RemoveText(SourceRange(StartLoc, EndLoc)));
   }
 
@@ -1221,6 +1240,15 @@ bool RewriteUtils::insertStringBeforeFunc(const FunctionDecl *FD,
   return !TheRewriter->InsertTextBefore(StartLoc, Str);
 }
 
+bool RewriteUtils::insertStringBeforeTemplateDecl(const TemplateDecl *D,
+                                                  const std::string &Str)
+{
+  SourceRange Range = D->getSourceRange();
+  SourceLocation StartLoc = Range.getBegin();
+  TransAssert(StartLoc.isValid() && "Invalid template decl StartLoc!");
+  return !TheRewriter->InsertTextBefore(StartLoc, Str);
+}
+
 bool RewriteUtils::replaceUnionWithStruct(const NamedDecl *ND)
 {
   SourceRange NDRange = ND->getSourceRange();
@@ -1309,6 +1337,18 @@ bool RewriteUtils::removeFieldDecl(const FieldDecl *FD)
   SourceRange Range = FD->getSourceRange();
   SourceLocation StartLoc = Range.getBegin();
   SourceLocation EndLoc = getEndLocationUntil(Range, ';');
+  SourceLocation CurlyEndLoc = getEndLocationUntil(Range, '}');
+
+  // handle cases like:
+  // struct {
+  //   int f <- no semicolon here
+  // };
+  const char *SemiPos = SrcManager->getCharacterData(EndLoc);
+  const char *CurlyPos = SrcManager->getCharacterData(CurlyEndLoc);
+  if (SemiPos > CurlyPos) {
+    EndLoc = CurlyEndLoc.getLocWithOffset(-1);
+  }
+
   return !(TheRewriter->RemoveText(SourceRange(StartLoc, EndLoc)));
 }
 
@@ -1582,3 +1622,20 @@ bool RewriteUtils::removeClassTemplateDecls(const ClassTemplateDecl *TmplD)
   return true;
 }
 
+bool RewriteUtils::replaceCXXMethodNameAfterQualifier(
+       const NestedNameSpecifierLoc *QualLoc,
+       const CXXMethodDecl *MD,
+       const std::string &NewName)
+{
+  SourceLocation EndLoc = QualLoc->getEndLoc();
+  const char *EndBuf = SrcManager->getCharacterData(EndLoc);
+  unsigned int Offset = 0;
+  while (isspace(*EndBuf) || (*EndBuf == ':')) {
+    EndBuf++;
+    Offset++;
+  }
+  EndLoc = EndLoc.getLocWithOffset(Offset);
+  TheRewriter->ReplaceText(EndLoc, MD->getNameAsString().size(), NewName);
+  return true;
+}
+
