@@ -1,6 +1,6 @@
 //===----------------------------------------------------------------------===//
 //
-// Copyright (c) 2012 The University of Utah
+// Copyright (c) 2012, 2013 The University of Utah
 // All rights reserved.
 //
 // This file is distributed under the University of Illinois Open Source
@@ -18,6 +18,7 @@
 
 #include "clang/AST/RecursiveASTVisitor.h"
 #include "clang/AST/ASTContext.h"
+#include "clang/Lex/Lexer.h"
 #include "clang/Basic/SourceManager.h"
 #include "llvm/Support/raw_ostream.h"
 #include "llvm/ADT/SmallString.h"
@@ -137,6 +138,9 @@ void Transformation::getTransErrorMsg(std::string &ErrorMsg)
   else if (TransError == TransNoValidParamsError) {
     ErrorMsg = "No valid parameters declarations exist!";
   }
+  else if (TransError == TransNoTextModificationError) {
+    ErrorMsg = "No modification to the transformed program!";
+  }
   else {
     TransAssert(0 && "Unknown transformation error!");
   }
@@ -200,10 +204,8 @@ const Expr *Transformation::getArrayBaseExprAndIdxs(
     unsigned int Idx = 0;
     llvm::APSInt Result;
     if (IdxE && IdxE->EvaluateAsInt(Result, *Context)) {
-      std::string IntStr = Result.toString(10);
-      std::stringstream TmpSS(IntStr);
-      if (!(TmpSS >> Idx))
-        TransAssert(0 && "Non-integer value!");
+      // this will truncate a possible uint64 value to uint32 value
+      Idx = (unsigned int)(*Result.getRawData());
     }
     BaseE = ASE->getBase()->IgnoreParenCasts();
     ASE = dyn_cast<ArraySubscriptExpr>(BaseE);
@@ -230,9 +232,15 @@ const Expr *Transformation::getInitExprFromBase(const Expr *BaseE,
 
   if (!InitE)
     return NULL;
+  // We don't have routine for CXXConstructExpr
+  if (dyn_cast<CXXConstructExpr>(InitE))
+    return NULL;
 
   const InitListExpr *ILE = dyn_cast<InitListExpr>(InitE);
-  TransAssert(ILE && "Bad InitListExpr!");
+  // not always the case we will have an InitListExpr, e.g.,
+  // RHS is a struct, or dereferencing a struct pointer, etc
+  if (!ILE)
+    return NULL;
   return getInitExprByIndex(Idxs, ILE);
 }
 
@@ -423,11 +431,7 @@ int Transformation::getIndexAsInteger(const Expr *E)
   if (!E->EvaluateAsInt(Result, *Context))
     TransAssert(0 && "Failed to Evaluate index!");
 
-  std::string IntStr = Result.toString(10);
-  std::stringstream TmpSS(IntStr);
-  if (!(TmpSS >> Idx))
-    TransAssert(0 && "Non-integer value!");
-
+  Idx = (int)(*Result.getRawData());
   return Idx;
 }
 
@@ -448,11 +452,78 @@ const Type* Transformation::getBaseType(const Type *T)
   return T;
 }
 
-const FunctionDecl *Transformation::lookupFunctionDecl(
+// Lookup a function decl from a top-level DeclContext
+// It's slow...
+const FunctionDecl *Transformation::lookupFunctionDeclInGlobal(
         DeclarationName &DName, const DeclContext *Ctx)
 {
   DeclContext::lookup_const_result Result = Ctx->lookup(DName);
-  for (DeclContext::lookup_const_iterator I = Result.first, E = Result.second;
+  for (DeclContext::lookup_const_iterator I = Result.begin(), E = Result.end();
+       I != E; ++I) {
+    if (const FunctionDecl *FD = dyn_cast<FunctionDecl>(*I)) {
+      return FD;
+    }
+
+    const FunctionTemplateDecl *TD = NULL;
+    if (const UsingShadowDecl *USD = dyn_cast<UsingShadowDecl>(*I)) {
+      TD = dyn_cast<FunctionTemplateDecl>(USD->getTargetDecl());
+    }
+    else {
+      TD = dyn_cast<FunctionTemplateDecl>(*I);
+    }
+    if (TD)
+      return TD->getTemplatedDecl();
+  }
+
+  for (DeclContext::decl_iterator I = Ctx->decls_begin(),
+       E = Ctx->decls_end(); I != E; ++I) {
+    if (const ClassTemplateDecl *ClassTemplate = 
+        dyn_cast<ClassTemplateDecl>(*I)) {
+      const CXXRecordDecl *CXXRD = ClassTemplate->getTemplatedDecl();
+      if (const FunctionDecl *FD = lookupFunctionDeclInGlobal(DName, CXXRD)) {
+        return FD;
+      }
+    }
+
+    const DeclContext *SubCtx = dyn_cast<DeclContext>(*I);
+    if (!SubCtx || dyn_cast<LinkageSpecDecl>(SubCtx))
+      continue;
+    
+    if (const FunctionDecl *FD = lookupFunctionDeclInGlobal(DName, SubCtx)) {
+      return FD;
+    }
+  }
+
+  return NULL;
+}
+
+const FunctionDecl *Transformation::lookupFunctionDeclFromBases(
+        DeclarationName &DName, const CXXRecordDecl *CXXRD)
+{
+  for (CXXRecordDecl::base_class_const_iterator I =
+       CXXRD->bases_begin(), E = CXXRD->bases_end(); I != E; ++I) {
+
+    const CXXBaseSpecifier *BS = I;
+    const Type *Ty = BS->getType().getTypePtr();
+    const CXXRecordDecl *Base = getBaseDeclFromType(Ty);
+    // it's not always the case we could resolve a base specifier, e.g.
+    // Ty is of DependentName
+    if (!Base)
+      continue;
+    const CXXRecordDecl *BaseDef = Base->getDefinition();
+    if (!BaseDef)
+      continue;
+    if (const FunctionDecl *FD = lookupFunctionDecl(DName, BaseDef))
+      return FD;
+  }
+  return NULL;
+}
+
+const FunctionDecl *Transformation::lookupFunctionDeclFromCtx(
+        DeclarationName &DName, const DeclContext *Ctx)
+{
+  DeclContext::lookup_const_result Result = Ctx->lookup(DName);
+  for (DeclContext::lookup_const_iterator I = Result.begin(), E = Result.end();
        I != E; ++I) {
     if (const FunctionDecl *FD = dyn_cast<FunctionDecl>(*I)) {
       return FD;
@@ -467,11 +538,44 @@ const FunctionDecl *Transformation::lookupFunctionDecl(
     }
     if (TD)
       return TD->getTemplatedDecl();
+
+    if (const UnresolvedUsingValueDecl *UUD = 
+        dyn_cast<UnresolvedUsingValueDecl>(*I)) {
+      const NestedNameSpecifier *NNS = UUD->getQualifier();
+      const DeclContext *Ctx = getDeclContextFromSpecifier(NNS);
+      if (!Ctx)
+        continue;
+      if (const FunctionDecl *FD = lookupFunctionDecl(DName, Ctx))
+        return FD;
+    }
+  }
+  return NULL;
+}
+
+const FunctionDecl *Transformation::lookupFunctionDecl(
+        DeclarationName &DName, const DeclContext *Ctx)
+{
+  if (const FunctionDecl *FD = lookupFunctionDeclFromCtx(DName, Ctx))
+    return FD;
+ 
+  // lookup base classes:
+  // this would be slow and may re-visit some Ctx. 
+  // Probably we should cache those Ctx(s) which have been visited
+  // to speedup lookup
+  if (Ctx->isRecord()) {
+    const RecordDecl *RD = dyn_cast<RecordDecl>(Ctx);
+    if (const CXXRecordDecl *CXXRD = dyn_cast<CXXRecordDecl>(RD)) {
+      if (const FunctionDecl *FD = lookupFunctionDeclFromBases(DName, CXXRD))
+        return FD;
+    }
   }
 
   for (DeclContext::udir_iterator I = Ctx->using_directives_begin(),
        E = Ctx->using_directives_end(); I != E; ++I) {
     const NamespaceDecl *ND = (*I)->getNominatedNamespace();
+    // avoid infinite recursion
+    if (ND->getLookupParent() == Ctx)
+      return NULL;
     if (const FunctionDecl *FD = lookupFunctionDecl(DName, ND))
       return FD;
   }
@@ -502,6 +606,16 @@ const DeclContext *Transformation::getDeclContextFromSpecifier(
         const Type *Ty = NNS->getAsType();
         if (const RecordType *RT = Ty->getAs<RecordType>())
           return RT->getDecl();
+        if (const TypedefType *TT = Ty->getAs<TypedefType>()) {
+          const TypedefNameDecl *TypeDecl = TT->getDecl();
+          const Type *UnderlyingTy = TypeDecl->getUnderlyingType().getTypePtr();
+          if (const RecordType *RT = UnderlyingTy->getAs<RecordType>())
+            return RT->getDecl();
+          if (const TemplateSpecializationType *TST =
+              UnderlyingTy->getAs<TemplateSpecializationType>()) {
+            return getBaseDeclFromTemplateSpecializationType(TST);
+          }
+        }
         break;
       }
       default:
@@ -521,11 +635,24 @@ const CXXRecordDecl *Transformation::getBaseDeclFromTemplateSpecializationType(
         const TemplateSpecializationType *TSTy)
 {
   TemplateName TplName = TSTy->getTemplateName();
-  const TemplateDecl *TplD = TplName.getAsTemplateDecl();
+  TemplateDecl *TplD = TplName.getAsTemplateDecl();
   TransAssert(TplD && "Invalid TemplateDecl!");
   NamedDecl *ND = TplD->getTemplatedDecl();
   TransAssert(ND && "Invalid NamedDecl!");
-  return dyn_cast<CXXRecordDecl>(ND);
+
+  if (TypedefNameDecl *TdefD = dyn_cast<TypedefNameDecl>(ND)) {
+    const Type *UnderlyingTy = TdefD->getUnderlyingType().getTypePtr();
+    const CXXRecordDecl *CXXRD = getBaseDeclFromType(UnderlyingTy);
+    TransAssert(CXXRD && "Invalid CXXRD from TypedefNameDecl!");
+    return CXXRD;
+  }
+
+  const CXXRecordDecl *CXXRD = dyn_cast<CXXRecordDecl>(ND);
+  TransAssert(CXXRD && "Invalid CXXRD!");
+  if (CXXRD->hasDefinition())
+    return CXXRD->getDefinition();
+  else
+    return CXXRD;
 }
 
 // This function could return NULL
@@ -550,18 +677,48 @@ const CXXRecordDecl *Transformation::getBaseDeclFromType(const Type *Ty)
   case Type::Elaborated: {
     const ElaboratedType *ETy = dyn_cast<ElaboratedType>(Ty);
     const Type *NamedT = ETy->getNamedType().getTypePtr();
-    if ( const TemplateSpecializationType *TSTy = 
-         dyn_cast<TemplateSpecializationType>(NamedT) ) {
-      Base = getBaseDeclFromTemplateSpecializationType(TSTy);
-    }
-    else if ( const TypedefType * Ty = dyn_cast<TypedefType>(NamedT) ){
-      Base = getBaseDeclFromType(Ty);
-    }
-    else {
-      Base = ETy->getAsCXXRecordDecl();
-    }
-    TransAssert(Base && "Bad base class type from ElaboratedType!");
-    return Base;
+    return getBaseDeclFromType(NamedT);
+  }
+
+  case Type::Paren: {
+    const ParenType *PT = dyn_cast<ParenType>(Ty);
+    const Type *InnerTy = PT->getInnerType().getTypePtr();
+    return getBaseDeclFromType(InnerTy);
+  }
+
+  case Type::Typedef: {
+    const TypedefType *TdefTy = dyn_cast<TypedefType>(Ty);
+    const TypedefNameDecl *TdefD = TdefTy->getDecl();
+    const Type *UnderlyingTy = TdefD->getUnderlyingType().getTypePtr();
+    return getBaseDeclFromType(UnderlyingTy);
+  }
+
+  case Type::Decltype: {
+    const DecltypeType *DT = dyn_cast<DecltypeType>(Ty);
+    const Type *UnderlyingTy = DT->getUnderlyingType().getTypePtr();
+    return getBaseDeclFromType(UnderlyingTy);
+  }
+
+  case Type::ConstantArray:
+  case Type::DependentSizedArray:
+  case Type::IncompleteArray:
+  case Type::VariableArray: { // fall-through
+    const ArrayType *AT = dyn_cast<ArrayType>(Ty);
+    const Type *ElemTy = AT->getElementType().getTypePtr();
+    return getBaseDeclFromType(ElemTy);
+  }
+
+  case Type::MemberPointer: {
+    const MemberPointerType *MPT = dyn_cast<MemberPointerType>(Ty);
+    const Type *PT = MPT->getPointeeType().getTypePtr();
+    return getBaseDeclFromType(PT);
+  }
+
+  case Type::SubstTemplateTypeParm: {
+    const SubstTemplateTypeParmType *TP = 
+      dyn_cast<SubstTemplateTypeParmType>(Ty);
+    const Type *ST = TP->getReplacementType().getTypePtr();
+    return getBaseDeclFromType(ST);
   }
 
   case Type::DependentName: {
@@ -577,27 +734,6 @@ const CXXRecordDecl *Transformation::getBaseDeclFromType(const Type *Ty)
     return NULL;
   }
 
-  case Type::Typedef: {
-    const TypedefType *TdefTy = dyn_cast<TypedefType>(Ty);
-    const TypedefNameDecl *TdefD = TdefTy->getDecl();
-    const Type *UnderlyingTy = TdefD->getUnderlyingType().getTypePtr();
-    if ( const TemplateSpecializationType *TSTy = 
-         dyn_cast<TemplateSpecializationType>(UnderlyingTy) ) {
-      Base = getBaseDeclFromTemplateSpecializationType(TSTy);
-    }
-    else if (dyn_cast<DependentNameType>(UnderlyingTy)) {
-      return NULL;
-    }
-    else if (dyn_cast<TemplateTypeParmType>(UnderlyingTy)) {
-      return NULL;
-    }
-    else {
-      Base = UnderlyingTy->getAsCXXRecordDecl();
-    }
-    TransAssert(Base && "Bad base class type from Typedef!");
-    return Base;
-  }
-
   case Type::TemplateTypeParm: {
     // Yet another case we might not know the base class, e.g.,
     // template<typename T1> 
@@ -606,6 +742,14 @@ const CXXRecordDecl *Transformation::getBaseDeclFromType(const Type *Ty)
     // };
     return NULL;
   }
+
+  case Type::Enum:
+  case Type::FunctionProto:
+  case Type::FunctionNoProto:
+  case Type::SubstTemplateTypeParmPack:
+  case Type::Builtin: // fall-through
+    return NULL;
+
   default:
     Base = Ty->getAsCXXRecordDecl();
     TransAssert(Base && "Bad base class type!");
@@ -659,6 +803,210 @@ unsigned Transformation::getNumCtorWrittenInitializers(
   return Num;
 }
 
+bool Transformation::isBeforeColonColon(TypeLoc &Loc)
+{
+  SourceLocation EndLoc = Loc.getEndLoc();
+  SourceLocation ColonColonLoc =
+      Lexer::findLocationAfterToken(EndLoc,
+                                    tok::coloncolon,
+                                    *SrcManager,
+                                    Context->getLangOpts(),
+                                    /*SkipTrailingWhitespaceAndNewLine=*/true);
+  return ColonColonLoc.isValid();
+}
+
+bool Transformation::replaceDependentNameString(const Type *Ty,
+                                                const TemplateArgument *Args,
+                                                unsigned NumArgs,
+                                                std::string &Str,
+                                                bool &Typename)
+{
+  TransAssert((Ty->getTypeClass() == Type::DependentName) &&
+              "Not DependentNameType!");
+
+  const DependentNameType *DNT = dyn_cast<DependentNameType>(Ty);
+  const IdentifierInfo *IdInfo = DNT->getIdentifier();
+  if (!IdInfo)
+    return false;
+  const NestedNameSpecifier *Specifier = DNT->getQualifier();
+  if (!Specifier)
+    return false;
+  const Type *DependentTy = Specifier->getAsType();
+  if (!DependentTy)
+    return false;
+
+  const TemplateTypeParmType *ParmTy = 
+    DependentTy->getAs<TemplateTypeParmType>();
+  if (!ParmTy)
+    return false;
+
+  unsigned Idx = ParmTy->getIndex();
+  TransAssert((Idx < NumArgs) && "Bad Parm Index!");
+  const TemplateArgument Arg = Args[Idx];
+  if (Arg.getKind() != TemplateArgument::Type)
+    return false;
+  QualType ArgQT = Arg.getAsType();
+  ArgQT.getAsStringInternal(Str, Context->getPrintingPolicy());
+  Str += "::";
+  Str += IdInfo->getName();
+  Typename = true;
+  return true;
+}
+
+bool Transformation::getTemplateTypeParmString(
+       const TemplateTypeParmType *ParmTy,
+       const TemplateArgument *Args,
+       unsigned NumArgs,
+       std::string &Str)
+{
+  unsigned Idx = ParmTy->getIndex();
+  // we could have default template args, skip this case for now
+  if (Idx >= NumArgs)
+    return false;
+  const TemplateArgument Arg = Args[Idx];
+  if (Arg.getKind() != TemplateArgument::Type)
+    return false;
+  QualType ArgQT = Arg.getAsType();
+  ArgQT.getAsStringInternal(Str, Context->getPrintingPolicy());
+  return true;
+}
+
+bool Transformation::getTypedefString(const StringRef &Name,
+                                      const CXXRecordDecl *CXXRD,
+                                      const TemplateArgument *Args,
+                                      unsigned NumArgs,
+                                      std::string &Str,
+                                      bool &Typename)
+{
+  Str = "";
+  for (DeclContext::decl_iterator I = CXXRD->decls_begin(),
+       E = CXXRD->decls_end(); I != E; ++I) {
+    const TypedefDecl *D = dyn_cast<TypedefDecl>(*I);
+    if (!D || (D->getNameAsString() != Name))
+      continue;
+    const Type *UnderlyingTy = D->getUnderlyingType().getTypePtr();
+    Type::TypeClass TC = UnderlyingTy->getTypeClass();
+    if (TC == Type::DependentName) {
+      if (replaceDependentNameString(UnderlyingTy, Args, 
+                                     NumArgs, Str, Typename))
+        return true;
+    }
+    else if (const TemplateTypeParmType *ParmTy = 
+             UnderlyingTy->getAs<TemplateTypeParmType>()) {
+      if (getTemplateTypeParmString(ParmTy, Args, NumArgs, Str))
+        return true;
+    }
+  }
+
+  // check base class
+  for (CXXRecordDecl::base_class_const_iterator I =
+       CXXRD->bases_begin(), E = CXXRD->bases_end(); I != E; ++I) {
+    const CXXBaseSpecifier *BS = I;
+    const Type *Ty = BS->getType().getTypePtr();
+    const CXXRecordDecl *Base = getBaseDeclFromType(Ty);
+    // it could happen if we have a dependent base specifier
+    if (!Base)
+      continue;
+    const CXXRecordDecl *BaseDef = Base->getDefinition();
+    if (!BaseDef)
+      continue;
+    if (getTypedefString(Name, BaseDef, Args, NumArgs, Str, Typename))
+      return true;
+  }
+
+  // TODO: really simplified lookup process, maybe need 
+  //       to check other decl context?
+  return false;
+}
+
+bool Transformation::getDependentNameTypeString(
+       const DependentNameType *DNT, std::string &Str, bool &Typename)
+{
+  const IdentifierInfo *IdInfo = DNT->getIdentifier();
+  if (!IdInfo)
+    return false;
+  const NestedNameSpecifier *Specifier = DNT->getQualifier();
+  if (!Specifier)
+    return false;
+  const Type *Ty = Specifier->getAsType();
+  if (!Ty)
+    return false;
+  const CXXRecordDecl *Base = getBaseDeclFromType(Ty);
+  if (!Base)
+    return false;
+  const CXXRecordDecl *BaseDef = Base->getDefinition();
+  if (!BaseDef)
+    return false;
+
+  unsigned NumArgs = 0;
+  const TemplateArgument *Args = NULL;
+  if (const TemplateSpecializationType *TST = 
+      Ty->getAs<TemplateSpecializationType>()) {
+    NumArgs = TST->getNumArgs();
+    Args = TST->getArgs();
+  }
+  return getTypedefString(IdInfo->getName(), 
+           BaseDef, Args, NumArgs, Str, Typename);
+}
+
+bool Transformation::getTypeString(const QualType &QT, 
+                                   std::string &Str,
+                                   bool &Typename)
+{
+  const Type *Ty = QT.getTypePtr();
+  Type::TypeClass TC = Ty->getTypeClass();
+
+  switch (TC) {
+  case Type::SubstTemplateTypeParm: {
+    const SubstTemplateTypeParmType *TP = 
+      dyn_cast<SubstTemplateTypeParmType>(Ty);
+    return getTypeString(TP->getReplacementType(), Str, Typename);
+  }
+
+  case Type::Elaborated: {
+    const ElaboratedType *ETy = dyn_cast<ElaboratedType>(Ty);
+    return getTypeString(ETy->getNamedType(), Str, Typename);
+  }
+  
+  case Type::Typedef: {
+    const TypedefType *TdefTy = dyn_cast<TypedefType>(Ty);
+    const TypedefNameDecl *TdefD = TdefTy->getDecl();
+    return getTypeString(TdefD->getUnderlyingType(), Str, Typename);
+  }
+
+  case Type::DependentName: {
+    const DependentNameType *DNT = dyn_cast<DependentNameType>(Ty);
+    return getDependentNameTypeString(DNT, Str, Typename);
+  }
+
+  case Type::Record:
+  case Type::Builtin: { // fall-through
+    QT.getAsStringInternal(Str, Context->getPrintingPolicy());
+    return true;
+  }
+
+  default:
+    return false;
+  }
+
+  TransAssert(0 && "Unreachable code!");
+  return false;
+}
+
+unsigned Transformation::getNumExplicitDecls(const CXXRecordDecl *CXXRD)
+{
+  const DeclContext *Ctx = dyn_cast<DeclContext>(CXXRD);
+  TransAssert(Ctx && "Invalid DeclContext!");
+
+  unsigned Num = 0;
+  for (DeclContext::decl_iterator I = Ctx->decls_begin(),
+       E = Ctx->decls_end(); I != E; ++I) {
+    if (!(*I)->isImplicit())
+      Num++;
+  }
+  return Num;
+}
+
 Transformation::~Transformation(void)
 {
   RewriteUtils::Finalize();
