@@ -1,6 +1,6 @@
 //===----------------------------------------------------------------------===//
 //
-// Copyright (c) 2012 The University of Utah
+// Copyright (c) 2012, 2013 The University of Utah
 // All rights reserved.
 //
 // This file is distributed under the University of Illinois Open Source
@@ -15,7 +15,7 @@
 #include <cstdlib>
 #include <cassert>
 #include "clang/AST/ASTConsumer.h"
-#include "clang/Rewrite/Rewriter.h"
+#include "clang/Rewrite/Core/Rewriter.h"
 #include "RewriteUtils.h"
 
 namespace clang {
@@ -36,6 +36,11 @@ namespace clang {
   class TemplateSpecializationType;
   class NamedDecl;
   class CXXConstructorDecl;
+  class TypeLoc;
+  class TemplateArgument;
+  class TemplateTypeParmType;
+  class DependentNameType;
+  class QualType;
 }
 
 typedef enum {
@@ -46,7 +51,8 @@ typedef enum {
   TransMaxClassesError,
   TransNoValidVarsError,
   TransNoValidFunsError,
-  TransNoValidParamsError
+  TransNoValidParamsError,
+  TransNoTextModificationError
 } TransformationError;
 
 class Transformation : public clang::ASTConsumer {
@@ -62,7 +68,8 @@ public:
       SrcManager(NULL),
       TransError(TransSuccess),
       DescriptionString(Desc),
-      RewriteHelper(NULL)
+      RewriteHelper(NULL),
+      Rewritten(false)
   {
     // Nothing to do
   }
@@ -165,6 +172,15 @@ protected:
   const clang::FunctionDecl *lookupFunctionDecl(
           clang::DeclarationName &DName, const clang::DeclContext *Ctx);
 
+  const clang::FunctionDecl *lookupFunctionDeclFromCtx(
+          clang::DeclarationName &DName, const clang::DeclContext *Ctx);
+
+  const clang::FunctionDecl *lookupFunctionDeclFromBases(
+          clang::DeclarationName &DName, const clang::CXXRecordDecl *CXXRD);
+
+  const clang::FunctionDecl *lookupFunctionDeclInGlobal(
+          clang::DeclarationName &DName, const clang::DeclContext *Ctx);
+
   const clang::DeclContext *getDeclContextFromSpecifier(
           const clang::NestedNameSpecifier *NNS);
 
@@ -173,12 +189,42 @@ protected:
   const clang::CXXRecordDecl *getBaseDeclFromType(const clang::Type *Ty);
 
   const clang::CXXRecordDecl *getBaseDeclFromTemplateSpecializationType(
-        const clang::TemplateSpecializationType *TSTy);
+          const clang::TemplateSpecializationType *TSTy);
 
   bool isParameterPack(const clang::NamedDecl *ND);
 
   unsigned getNumCtorWrittenInitializers(const clang::CXXConstructorDecl &Ctor);
 
+  bool isBeforeColonColon(clang::TypeLoc &Loc);
+
+  bool getTypeString(const clang::QualType &QT, 
+                     std::string &Str,
+                     bool &Typename);
+
+  bool getTypedefString(const llvm::StringRef &Name,
+                        const clang::CXXRecordDecl *CXXRD,
+                        const clang::TemplateArgument *Args,
+                        unsigned NumArgs,
+                        std::string &Str,
+                        bool &Typename);
+
+  bool getDependentNameTypeString(const clang::DependentNameType *DNT,
+                                  std::string &Str,
+                                  bool &Typename);
+
+  bool replaceDependentNameString(const clang::Type *Ty,
+                                  const clang::TemplateArgument *Args,
+                                  unsigned NumArgs,
+                                  std::string &Str,
+                                  bool &Typename);
+
+  bool getTemplateTypeParmString(const clang::TemplateTypeParmType *ParmTy,
+                                const clang::TemplateArgument *Args,
+                                unsigned NumArgs,
+                                std::string &Str);
+
+  unsigned getNumExplicitDecls(const clang::CXXRecordDecl *CXXRD);
+
   const std::string Name;
 
   int TransformationCounter;
@@ -198,6 +244,8 @@ protected:
   std::string DescriptionString;
 
   RewriteUtils *RewriteHelper;
+
+  bool Rewritten;
 };
 
 class TransNameQueryVisitor;
