@@ -1,6 +1,6 @@
 //===----------------------------------------------------------------------===//
 //
-// Copyright (c) 2012 The University of Utah
+// Copyright (c) 2012, 2013 The University of Utah
 // All rights reserved.
 //
 // This file is distributed under the University of Illinois Open Source
@@ -15,6 +15,7 @@
 #include "UnionToStruct.h"
 
 #include "clang/AST/RecursiveASTVisitor.h"
+#include "clang/AST/ASTContext.h"
 #include "clang/Basic/SourceManager.h"
 
 #include "TransformationManager.h"
@@ -390,6 +391,9 @@ void UnionToStruct::addOneDeclarator(const DeclaratorDecl *DD, const Type *T)
   if (!T->isUnionType())
     return;
 
+  if (T->getTypeClass() == Type::Typedef)
+    return;
+
   const RecordType *RDTy = T->getAsUnionType();
   TransAssert(RDTy && "Bad RecordType!");
 
@@ -417,8 +421,7 @@ void UnionToStruct::addOneRecord(const RecordDecl *RD)
 
 UnionToStruct::~UnionToStruct(void)
 {
-  if (CollectionVisitor)
-    delete CollectionVisitor;
+  delete CollectionVisitor;
 
   for (RecordDeclToDeclaratorDeclMap::iterator I = RecordToDeclarator.begin(),
        E = RecordToDeclarator.end(); I != E; ++I) {
