@@ -1,6 +1,6 @@
 ## -*- mode: Perl -*-
 ##
-## Copyright (c) 2012 The University of Utah
+## Copyright (c) 2012, 2013 The University of Utah
 ## All rights reserved.
 ##
 ## This file is distributed under the University of Illinois Open Source
@@ -14,9 +14,11 @@ use warnings;
 
 use Exporter::Lite;
 
-@EXPORT      = qw(read_file write_file $OK $STOP
+@EXPORT      = qw(read_file write_file $OK $STOP $VERBOSE
                   $replace_cont replace_aux runit $matched);
 
+$VERBOSE = 0;
+
 $OK = 999999;
 $STOP = 111333;
 
@@ -34,6 +36,7 @@ sub runit ($) {
     return ($? >> 8);
 }
 
+# utility code to help us replace the nth occurrence of a pattern
 $replace_cont = 0;
 $matched = 0;
 sub replace_aux ($$$) { 
